// The Developer of the Original Code is Aki Goto <harpy@lily.freemail.ne.jp>
// to and including version 1.4

// Contributor: George A. Booth <booth.g@gmail.com>
// versions 1.5, 1.6

// Copyright licence is MPL 1.1/GPL 2.0/LGPL 2.1



var	rimwheel_enabled;
var	rimwheel_rimsize;
var	rimwheel_pageratio;
var	rimwheel_scrollsize;
var	rimwheel_debug;

var	enable_topleft;
var	enable_topmiddle;
var	enable_topright;
var	enable_middleleft;
var	enable_middleright;
var	enable_bottomleft;
var	enable_bottommiddle;
var	enable_bottomright;

var	rimwheel_x = 0;
var	rimwheel_y = 0;

var	debug = false;

rimwheel_load_settings();

window.addEventListener("mousemove", rimwheel_move_handler, false);
window.addEventListener("DOMMouseScroll", rimwheel_wheel_handler, false);

function rimwheel_load_settings()
{

// set bit masks

	const topleft=128;
	const topmiddle=64;
	const topright=32;
	const middleleft=16;
	const middleright=8;
	const bottomleft=4;
	const bottommiddle=2;
	const bottomright=1;

	var prefs = Components.classes["@mozilla.org/preferences-service;1"]
		.getService(Components.interfaces.nsIPrefService)
		.getBranch("extensions.rimwheel.");

	var enable_grid;

	rimwheel_enabled = rimwheel_getBoolPref(prefs, "enabled", true);
	rimwheel_rimsize = rimwheel_getIntPref(prefs, "rimsize", 64);
	rimwheel_pageratio = rimwheel_getCharPref(prefs, "pageratio", 0.9);
	rimwheel_scrollsize = rimwheel_getIntPref(prefs, "scrollsize", 48);

	enable_grid = rimwheel_getIntPref( prefs, "grid", 255 );

// unpack the grid preference - I punched fortran decks, I love this. - George

	enable_topleft = ( enable_grid & topleft ) != 0;
	enable_topmiddle = ( enable_grid & topmiddle ) != 0;
	enable_topright = ( enable_grid & topright ) != 0;
	enable_middleleft = ( enable_grid & middleleft ) != 0;
	enable_middleright = ( enable_grid & middleright ) != 0;
	enable_bottomleft = ( enable_grid & bottomleft ) != 0;
	enable_bottommiddle = ( enable_grid & bottommiddle ) != 0;
	enable_bottomright = ( enable_grid & bottomright ) != 0;
	
}

function rimwheel_getBoolPref(prefs, name, d)
{
	if (prefs.prefHasUserValue(name))
		return prefs.getBoolPref(name);
	else
		return d;
}

function rimwheel_getIntPref(prefs, name, d)
{
	if (prefs.prefHasUserValue(name))
		return prefs.getIntPref(name);
	else
		return d;
}

function rimwheel_getCharPref(prefs, name, d)
{
	if (prefs.prefHasUserValue(name))
		return prefs.getCharPref(name);
	else
		return d;
}

RimwheelPrefsObserver.prototype =
{
	observe:
		function(subject, topic, data) {
			rimwheel_load_settings();
		}
}

var rimwheel_prefs_observer = new RimwheelPrefsObserver();

function RimwheelPrefsObserver()
{
	try {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService)
			.getBranch(null);
		var pbi = prefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.addObserver("extensions.rimwheel", this, false);
	} catch (e) {
		alert(e);
	}
}

function rimwheel_move_handler(e)
{
	rimwheel_x = e.screenX - gBrowser.boxObject.screenX;
	rimwheel_y = e.screenY - gBrowser.boxObject.screenY;
}

function rimwheel_wheel_handler(e)
{
	if (debug) 
		alert("e(x,y)=("+e.screenX+","+e.screenY+"), "
			+"Rimwheel(x,y)=("+rimwheel_x+","+rimwheel_y+"), "
			+"(ClientX,ClientY) = ("+e.clientX+","+e.clientY+"), ");

	if (!rimwheel_enabled) return;

	var ix, iy;

	if (rimwheel_x < rimwheel_rimsize) ix = 0;
	else if (rimwheel_x < gBrowser.boxObject.width - rimwheel_rimsize) ix = 1;
	else ix = 2;

	if (rimwheel_y < rimwheel_rimsize) iy = 0;
	else if (rimwheel_y < gBrowser.boxObject.height - rimwheel_rimsize) iy = 1;
	else iy = 2;

	switch (iy * 3 + ix) {

		case 0: // upper left
			if ( enable_topleft) {
				if (e.detail < 0)	{ // up
					BrowserStop();
					}
				else	{ // down
					BrowserReload();
					};
				e.preventDefault();
				};
			return;

		case 1: // upper
			if ( enable_topmiddle ) {
				if (e.detail < 0) // up
					gBrowser.tabContainer.advanceSelectedTab(true ? 1 : -1, true);
				else // down
					gBrowser.tabContainer.advanceSelectedTab(false ? 1 : -1, true);
				e.preventDefault();
				};
			return;

		case 2: // upper right
			if ( enable_topright) { 
				if (e.detail < 0)  {// up
					if (gBrowser.tabContainer.childNodes.length > 1 )
						gBrowser.removeCurrentTab();
					 else	; // nothing
					 }
				else// down
					delayedOpenTab();
				e.preventDefault();
				};
			return;

		case 3: // left
			if ( enable_middleleft) {
				if (e.detail < 0) // up
					BrowserBack();
				else // down
					BrowserForward();
				e.preventDefault();
				};
			return;

		case 4: // center
			return; // do default operation

		case 5: // right
			if ( enable_middleright) {
				if (e.detail < 0) { // up
					var w = e.target.ownerDocument.defaultView;
					w.scrollBy(0, -w.innerHeight * rimwheel_pageratio);
					}
				else { // down
					var w = e.target.ownerDocument.defaultView;
					w.scrollBy(0, w.innerHeight * rimwheel_pageratio);
					};
				e.preventDefault();
				};
			return;

		case 6: // lower left
			if ( enable_bottomleft ) {
				if (e.detail < 0) // up
					getWebNavigation().gotoIndex(0);
				else // down
					getWebNavigation().gotoIndex( getWebNavigation().sessionHistory.count - 1 );
				e.preventDefault();
				};
			return;

		case 7: // lower
			if ( enable_bottommiddle ) {
				if (e.detail < 0) // up
					e.target.ownerDocument.defaultView.scrollBy(-rimwheel_scrollsize, 0);
				else // down
					e.target.ownerDocument.defaultView.scrollBy(rimwheel_scrollsize, 0);
				e.preventDefault();
				};
			return;

		case 8: // lower right
			if ( enable_bottomright ) {
				if (e.detail < 0) { // up
					var w = e.target.ownerDocument.defaultView;
					w.scrollTo(w.scrollX, 0);
					}
				else { // down
					var w = e.target.ownerDocument.defaultView;
					w.scrollTo(w.scrollX, w.scrollMaxY);
					};
				e.preventDefault();
				};
			return;
	};
}

function rimwheel_settings()
{
	window.openDialog("chrome://rimwheel/content/settings.xul");

}
